#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Collections.Generic;
using System.Windows.Forms;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    [Description("Enter the description of your new custom indicator here")]
    public class mrZones100 : Indicator
    {
        #region Variables
        // Wizard generated variables
        private int priorCandles = 3; // Default setting for PriorCandles
        private int opacity = 100; // Default setting for Opacity
		private int inZoneOpacity = 100; 
		private int narrowZoneOpacity = 100; 
		private Color wideBarUpColor = Color.Blue;
		private Color wideBarDownColor = Color.Magenta;
		private string alertFile = @"Alert1.wav";
		private string wbAlertFile = @"Alert2.wav";
		private bool showNarrowZones = false;
        // User defined variables (add any user defined variables below)
		private DataSeries bodyRange;
		private List<mrZone> zones = new List<mrZone>();
		
		private System.Windows.Forms.Button btnText = null, btnNarrow = null;
		
		[XmlIgnore()]		
		public double LastPrice = 0;
		
        #endregion
		
		private class mrZone
		{
			public int StartBar, EndBar, Direction;
			public double High, Low;
			public bool Active = false, Narrow;
			
			private bool inZone = false;				
			private bool wasInZone = false;
			private StringFormat sf;	
			private int lastAlertBar = -1;
			private mrZones100 i;
			public mrZone(mrZones100 indicator, int startBar, double high, double low, int direction, bool narrow)
			{
				i=indicator;
				StartBar=startBar;
				High=high;
				Low=low;
				EndBar=startBar+1;
				Direction=direction;
				Active=true;
				Narrow=narrow;
				
				sf = new StringFormat();
				sf.Alignment=StringAlignment.Far;
				sf.LineAlignment=StringAlignment.Center;
			}
			
			public bool Visible(double min, double max)
			{				
				int firstBar = i.FirstBarIndexPainted;
				int lastBar = i.LastBarIndexPainted;
				
				bool c1 = StartBar<=lastBar && EndBar>=firstBar;
				bool c2 = High>min && Low<max;
				
				return c1 && c2;
			}
			
			public void Plot(Graphics graphics)
			{
				if (!i.NZEnabled && Narrow)
					return;
				
				int addBars = Active ? 4 : 0;
				
				int x = i.ChartControl.GetXByBarIdx(i.BarsArray[0],StartBar);
				int x2 = i.ChartControl.GetXByBarIdx(i.BarsArray[0],EndBar+addBars);
				int width = Math.Abs(x-x2);
				
				int y = i.ChartControl.GetYByValue(i,High);
				int y2 = i.ChartControl.GetYByValue(i,Low);
				int height = Math.Abs(y-y2);
				
				bool inZone = i.LastPrice<=High && i.LastPrice>=Low;
				Plot plot = Active && inZone ? i.Plots[1] : Narrow ? i.Plots[2] : i.Plots[0];
				int op = Active && inZone ? i.InZoneOpacity : Narrow ? i.NarrowZoneOpacity : i.Opacity;
				
				Rectangle r = new Rectangle(x,y,width,height);				
				SolidBrush brush = new SolidBrush(Color.FromArgb(op,plot.Pen.Color));
				graphics.FillRectangle(brush,r);
				
				if (i.TextEnabled)
				{
					brush = new SolidBrush(i.ChartControl.ChartStyle.Pen.Color);								
					DateTime startTime = i.Bars.GetTime(StartBar-1);
					string str = startTime.ToString("MM/dd/yy HH:mm");
					Font fnt = i.ChartControl.Font;
					Font font = new Font(fnt.FontFamily.ToString(),fnt.Size-1,fnt.Style);
					y = (int)Math.Round((decimal)(y+y2)/2);
					graphics.DrawString(str,font,brush,x2,y,sf);
				}
			}
			
			public void CheckAlerts()
			{
				if (i.CurrentBar<=lastAlertBar)
					return;
				
				wasInZone = inZone;
				inZone = i.LastPrice>=Low && i.LastPrice<=High;
				
				if ((wasInZone && !inZone) || (!wasInZone && inZone))
				{
					i.PlaySound(i.ZoneAlertFile);
					lastAlertBar=i.CurrentBar;
				}
			}
		}

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Green), PlotStyle.Line, "Zones"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "InZones"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Blue), PlotStyle.Line, "NarrowZones"));
            Overlay				= true;
        }
		
		protected override void OnStartUp()
		{
			bodyRange = new DataSeries(this);
			
			if (ChartControl!=null)
			{
				System.Windows.Forms.Control[] controls = ChartControl.Controls.Find("tsrTool", false);
				
				if (controls.Length > 0) 
				{				
					int btnPadding = 4;
					int btnHeight = 20;
					int btnWidth = 20;	
					
					int x=ChartControl.Left+btnPadding;
					int y=ChartControl.Top+btnPadding;
					
					btnText = new System.Windows.Forms.Button();										
					btnText.BackColor=Color.Red;    
					btnText.Text = "T";
					btnText.Click += btnText_Click;
					btnText.SetBounds(x,y,btnWidth,btnHeight);
					
					y+=btnHeight+btnPadding;
					btnNarrow = new System.Windows.Forms.Button();										
					btnNarrow.BackColor=showNarrowZones ? Color.Lime : Color.Red;    
					btnNarrow.Text = "Z";
					btnNarrow.Click += btnNarrow_Click;
					btnNarrow.SetBounds(x,y,btnWidth,btnHeight);	
					
					ChartControl.ChartPanel.Controls.Add(btnText);
					ChartControl.ChartPanel.Controls.Add(btnNarrow);	
				}
			}
		}

        protected override void OnBarUpdate()
        {
			double bodyHigh = Math.Max(Open[0],Close[0]);
			double bodyLow = Math.Min(Open[0],Close[0]);
			
            bodyRange[0] = bodyHigh-bodyLow;
			
			if (CurrentBar<priorCandles+2)
				return;
			
			if (bodyRange[0]>MAX(bodyRange,priorCandles)[1])
			{
				Color c = Close[0]>Open[0] ? wideBarUpColor : wideBarDownColor;
				BarColorSeries[0] = c;
				PlaySound(WideBarAlertFile);
			}
			
			if (bodyRange[1]>MAX(bodyRange,priorCandles)[2])
			{				
				if (Low[0]>High[2])
				{
					mrZone z = new mrZone(this,CurrentBar,Low[0],High[2],1,false);
					zones.Add(z);
				}
				else if (Low[2]>High[0])
				{
					mrZone z = new mrZone(this,CurrentBar,Low[2],High[0],-1,false);
					zones.Add(z);
				}
			}
			else
			{
				if (Low[0]>High[2])
				{
					mrZone z = new mrZone(this,CurrentBar,Low[0],High[2],1,true);
					zones.Add(z);
				}
				else if (Low[2]>High[0])
				{
					mrZone z = new mrZone(this,CurrentBar,Low[2],High[0],-1,true);
					zones.Add(z);
				}
			}
			
			List<mrZone> activeZones = zones.FindAll(delegate(mrZone z){return z.Active;});
			foreach (mrZone z in activeZones)
			{
				if (z.Direction==1)
				{
					if (Low[0]<z.Low)
						z.Active=false;
					z.EndBar=CurrentBar;										
				}
				else
				{
					if (High[0]>z.High)
						z.Active=false;
					z.EndBar=CurrentBar;
				}				
			}
        }
		
		protected override void OnMarketData(MarketDataEventArgs e)
		{
			if (e.MarketDataType!=MarketDataType.Last)
				return;
			
			LastPrice=e.Price;
			List<mrZone> activeZones = zones.FindAll(delegate(mrZone z){return z.Active;});
			foreach (mrZone z in activeZones)
				z.CheckAlerts();
		}
		
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			List<mrZone> toPlot = zones.FindAll(delegate(mrZone z){return z.Visible(min,max);});
			foreach (mrZone z in toPlot)
				z.Plot(graphics);
		}
		
		protected override void OnTermination()
		{	
			if (ChartControl!=null)	
			{
				if (btnText!=null)
					btnText.Dispose();
				if (btnNarrow!=null)
					btnNarrow.Dispose();				
			}
		}	
		
		private void btnText_Click(object sender, EventArgs e)
		{
			TextEnabled=!TextEnabled;
			ChartControl.ChartPanel.Invalidate();
		}
		
		private void btnNarrow_Click(object sender, EventArgs e)
		{
			NZEnabled=!NZEnabled;
			ChartControl.ChartPanel.Invalidate();
		}
		
		[XmlIgnore()]
		[Browsable(false)]
		public bool TextEnabled
		{
			get { return btnText.BackColor==Color.Lime; }
			set { btnText.BackColor = value ? Color.Lime : Color.Red; }
		}
		
		[XmlIgnore()]
		[Browsable(false)]
		public bool NZEnabled
		{
			get { return btnNarrow.BackColor==Color.Lime; }
			set { btnNarrow.BackColor = value ? Color.Lime : Color.Red; }
		}
		
        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Zones
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries InZones
        {
            get { return Values[1]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int PriorCandles
        {
            get { return priorCandles; }
            set { priorCandles = Math.Max(1, value); }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Opacity
        {
            get { return opacity; }
            set { opacity = Math.Min(255,Math.Max(0, value)); }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public int InZoneOpacity
        {
            get { return inZoneOpacity; }
            set { inZoneOpacity = Math.Min(255,Math.Max(0, value)); }
        }		
		
		[Description("")]
        [GridCategory("Parameters")]
        public int NarrowZoneOpacity
        {
            get { return narrowZoneOpacity; }
            set { narrowZoneOpacity = Math.Min(255,Math.Max(0, value)); }
        }		
		
		[Description("")]
        [GridCategory("Parameters")]
        public string ZoneAlertFile
        {
            get { return alertFile; }
            set { alertFile = value; }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public string WideBarAlertFile
        {
            get { return wbAlertFile; }
            set { wbAlertFile = value; }
        }
		
		[Description("")]
        [GridCategory("Parameters")]
        public bool ShowNarrowZones
        {
            get { return showNarrowZones; }
            set { showNarrowZones = value; }
        }
		
		[XmlIgnore()]
		[Description("")]
		[GridCategory("Parameters")]		
		public Color WideBarUpColor
		{
			get { return wideBarUpColor; }
			set { wideBarUpColor = value; }
		}
				
		[Browsable(false)]
		public string WideBarUpColorS
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(wideBarUpColor); }
			set { wideBarUpColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
		[Description("")]
		[GridCategory("Parameters")]		
		public Color WideBarDownColor
		{
			get { return wideBarDownColor; }
			set { wideBarDownColor = value; }
		}
				
		[Browsable(false)]
		public string WideBarDownColorS
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(wideBarDownColor); }
			set { wideBarDownColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private mrZones100[] cachemrZones100 = null;

        private static mrZones100 checkmrZones100 = new mrZones100();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public mrZones100 mrZones100(int inZoneOpacity, int narrowZoneOpacity, int opacity, int priorCandles, bool showNarrowZones, string wideBarAlertFile, Color wideBarDownColor, Color wideBarUpColor, string zoneAlertFile)
        {
            return mrZones100(Input, inZoneOpacity, narrowZoneOpacity, opacity, priorCandles, showNarrowZones, wideBarAlertFile, wideBarDownColor, wideBarUpColor, zoneAlertFile);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public mrZones100 mrZones100(Data.IDataSeries input, int inZoneOpacity, int narrowZoneOpacity, int opacity, int priorCandles, bool showNarrowZones, string wideBarAlertFile, Color wideBarDownColor, Color wideBarUpColor, string zoneAlertFile)
        {
            if (cachemrZones100 != null)
                for (int idx = 0; idx < cachemrZones100.Length; idx++)
                    if (cachemrZones100[idx].InZoneOpacity == inZoneOpacity && cachemrZones100[idx].NarrowZoneOpacity == narrowZoneOpacity && cachemrZones100[idx].Opacity == opacity && cachemrZones100[idx].PriorCandles == priorCandles && cachemrZones100[idx].ShowNarrowZones == showNarrowZones && cachemrZones100[idx].WideBarAlertFile == wideBarAlertFile && cachemrZones100[idx].WideBarDownColor == wideBarDownColor && cachemrZones100[idx].WideBarUpColor == wideBarUpColor && cachemrZones100[idx].ZoneAlertFile == zoneAlertFile && cachemrZones100[idx].EqualsInput(input))
                        return cachemrZones100[idx];

            lock (checkmrZones100)
            {
                checkmrZones100.InZoneOpacity = inZoneOpacity;
                inZoneOpacity = checkmrZones100.InZoneOpacity;
                checkmrZones100.NarrowZoneOpacity = narrowZoneOpacity;
                narrowZoneOpacity = checkmrZones100.NarrowZoneOpacity;
                checkmrZones100.Opacity = opacity;
                opacity = checkmrZones100.Opacity;
                checkmrZones100.PriorCandles = priorCandles;
                priorCandles = checkmrZones100.PriorCandles;
                checkmrZones100.ShowNarrowZones = showNarrowZones;
                showNarrowZones = checkmrZones100.ShowNarrowZones;
                checkmrZones100.WideBarAlertFile = wideBarAlertFile;
                wideBarAlertFile = checkmrZones100.WideBarAlertFile;
                checkmrZones100.WideBarDownColor = wideBarDownColor;
                wideBarDownColor = checkmrZones100.WideBarDownColor;
                checkmrZones100.WideBarUpColor = wideBarUpColor;
                wideBarUpColor = checkmrZones100.WideBarUpColor;
                checkmrZones100.ZoneAlertFile = zoneAlertFile;
                zoneAlertFile = checkmrZones100.ZoneAlertFile;

                if (cachemrZones100 != null)
                    for (int idx = 0; idx < cachemrZones100.Length; idx++)
                        if (cachemrZones100[idx].InZoneOpacity == inZoneOpacity && cachemrZones100[idx].NarrowZoneOpacity == narrowZoneOpacity && cachemrZones100[idx].Opacity == opacity && cachemrZones100[idx].PriorCandles == priorCandles && cachemrZones100[idx].ShowNarrowZones == showNarrowZones && cachemrZones100[idx].WideBarAlertFile == wideBarAlertFile && cachemrZones100[idx].WideBarDownColor == wideBarDownColor && cachemrZones100[idx].WideBarUpColor == wideBarUpColor && cachemrZones100[idx].ZoneAlertFile == zoneAlertFile && cachemrZones100[idx].EqualsInput(input))
                            return cachemrZones100[idx];

                mrZones100 indicator = new mrZones100();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.InZoneOpacity = inZoneOpacity;
                indicator.NarrowZoneOpacity = narrowZoneOpacity;
                indicator.Opacity = opacity;
                indicator.PriorCandles = priorCandles;
                indicator.ShowNarrowZones = showNarrowZones;
                indicator.WideBarAlertFile = wideBarAlertFile;
                indicator.WideBarDownColor = wideBarDownColor;
                indicator.WideBarUpColor = wideBarUpColor;
                indicator.ZoneAlertFile = zoneAlertFile;
                Indicators.Add(indicator);
                indicator.SetUp();

                mrZones100[] tmp = new mrZones100[cachemrZones100 == null ? 1 : cachemrZones100.Length + 1];
                if (cachemrZones100 != null)
                    cachemrZones100.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cachemrZones100 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.mrZones100 mrZones100(int inZoneOpacity, int narrowZoneOpacity, int opacity, int priorCandles, bool showNarrowZones, string wideBarAlertFile, Color wideBarDownColor, Color wideBarUpColor, string zoneAlertFile)
        {
            return _indicator.mrZones100(Input, inZoneOpacity, narrowZoneOpacity, opacity, priorCandles, showNarrowZones, wideBarAlertFile, wideBarDownColor, wideBarUpColor, zoneAlertFile);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.mrZones100 mrZones100(Data.IDataSeries input, int inZoneOpacity, int narrowZoneOpacity, int opacity, int priorCandles, bool showNarrowZones, string wideBarAlertFile, Color wideBarDownColor, Color wideBarUpColor, string zoneAlertFile)
        {
            return _indicator.mrZones100(input, inZoneOpacity, narrowZoneOpacity, opacity, priorCandles, showNarrowZones, wideBarAlertFile, wideBarDownColor, wideBarUpColor, zoneAlertFile);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.mrZones100 mrZones100(int inZoneOpacity, int narrowZoneOpacity, int opacity, int priorCandles, bool showNarrowZones, string wideBarAlertFile, Color wideBarDownColor, Color wideBarUpColor, string zoneAlertFile)
        {
            return _indicator.mrZones100(Input, inZoneOpacity, narrowZoneOpacity, opacity, priorCandles, showNarrowZones, wideBarAlertFile, wideBarDownColor, wideBarUpColor, zoneAlertFile);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.mrZones100 mrZones100(Data.IDataSeries input, int inZoneOpacity, int narrowZoneOpacity, int opacity, int priorCandles, bool showNarrowZones, string wideBarAlertFile, Color wideBarDownColor, Color wideBarUpColor, string zoneAlertFile)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.mrZones100(input, inZoneOpacity, narrowZoneOpacity, opacity, priorCandles, showNarrowZones, wideBarAlertFile, wideBarDownColor, wideBarUpColor, zoneAlertFile);
        }
    }
}
#endregion
